/***********************************************************************************
The program verifies the following statement: if P=(A,B) is an l-divided set with |A|=5, |B|=6,
A not in convex position, then there is an l-divided 5-hole in P.
************************************************************************************/

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
using namespace std;

const int MA = 5; // |A|
const int NA = MA*(MA-1)*(MA-2)/6; // (|A| choose 3)
const int numberOfConfA = 54; // number of signature functions for A of size 6, not in convex position
int confA[numberOfConfA][NA]; // list of  signature functions for A of size 6, not in convex position
int indexA[MA][MA][MA]; // indexation of triples of points of A
int reverse_indexA[NA][3]; // reverse indexation of triples of points of A
char const* filenameA  = "./A5Nonconvex.txt"; // input file for A


const int MB = 6; // |B|
const int NB = MB*(MB-1)*(MB-2)/6; // (|B| choose 3)
const int numberOfConfB = 908; // number of signature functions for B of size 5
int confB[numberOfConfB][NB]; // list of signature functions for B of size 5
int indexB[MB][MB][MB]; // indexation of triples of points of B
int reverse_indexB[NB][3]; // reverse indexation of triples of points of B
char const* filenameB  = "./B6.txt"; // input file for B


const int M = MA + MB; // |P|=|A|+|B|
const int N=M*(M-1)*(M-2)/6; // (|P| choose 3)
int index[M][M][M]; // indexation of triples of points of P
int reverse_index[N][3]; // reverse indexation of triples of points of P

int x[N]; // an auxiliary array
const int start = 0; // signature functions for A to be searched from "start" to "end" (for parallel executions), 0 <= start < end <= numberOfConfA
const int end = numberOfConfA;


// returns 1 iff q does not lie inside an l-divided 5-hole p0,...,p4 in the given (partial) signature function in P
// returns -1 iff p0,...,p4 is not an l-divided 5-hole in P (either a point q of P is inside or p0,...,p4 is not l-divided or p0,...,p4 are not in convex position)
// returns 0 iff it is not possible to determine whether q lies inside or whether p0,...,p4 are in convex position
// it is assumed that the x-coordinates satisfy x(p0) < ... < x(p4)
int divided5HoleCheck(int array[N], int p0, int p1, int p2, int p3, int p4, int q) {
	if(p0 < MA && p4 >= MA) {
		if(array[index[p0][p1][p2]] == 0 || array[index[p0][p1][p3]] == 0 || array[index[p0][p1][p4]] == 0 || array[index[p0][p2][p3]] == 0 || array[index[p0][p2][p4]] == 0 || array[index[p0][p3][p4]] == 0 || array[index[p1][p2][p3]] == 0 || array[index[p1][p2][p4]] == 0 || array[index[p1][p3][p4]] == 0 || array[index[p2][p3][p4]] == 0 ) {
			return 0;
		} else {
			if(array[index[p0][p1][p2]] == array[index[p1][p2][p3]] && array[index[p1][p2][p3]] == array[index[p2][p3][p4]]) {
				// 1,2,3 UP
				if(p0 < q && q < p1) {
					if(array[index[p0][q][p1]] == 0 || array[index[p0][q][p4]] == 0) {
						return 0;
					} else if(array[index[p0][q][p1]] != 0 && array[index[p0][q][p1]] == -array[index[p0][q][p4]]) {
						return -1;
					} else {
						return 1;
					}	
				} else if(p1 < q && q < p2) {
					if(array[index[p1][q][p2]] == 0 || array[index[p0][q][p4]] == 0) {
						return 0;
					} else if(array[index[p1][q][p2]] != 0 && array[index[p1][q][p2]] == -array[index[p0][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p2 < q && q < p3) {
					if(array[index[p2][q][p3]] == 0 || array[index[p0][q][p4]] == 0) {
						return 0;
					} else if(array[index[p2][q][p3]] != 0 && array[index[p2][q][p3]] == -array[index[p0][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p3 < q && q < p4) {
					if(array[index[p3][q][p4]] == 0 || array[index[p0][q][p4]] == 0) {
						return 0;
					} else if(array[index[p3][q][p4]] != 0 && array[index[p3][q][p4]] == -array[index[p0][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else {
					return 1;
				}
			} else if(array[index[p0][p1][p4]] == -array[index[p0][p2][p3]] && array[index[p0][p2][p3]] == array[index[p2][p3][p4]]) {
				// 2,3 UP, 1 DOWN
				if(p0 < q && q < p1) {
					if(array[index[p0][q][p1]] == 0 || array[index[p0][q][p2]] == 0) {
						return 0;
					} else if(array[index[p0][q][p1]] != 0 && array[index[p0][q][p1]] == -array[index[p0][q][p2]]) {
						return -1;
					} else {
						return 1;
					}	
				} else if(p1 < q && q < p2) {
					if(array[index[p0][q][p2]] == 0 || array[index[p1][q][p4]] == 0) {
						return 0;
					} else if(array[index[p0][q][p2]] != 0 && array[index[p0][q][p2]] == -array[index[p1][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p2 < q && q < p3) {
					if(array[index[p2][q][p3]] == 0 || array[index[p1][q][p4]] == 0) {
						return 0;
					} else if(array[index[p2][q][p3]] != 0 && array[index[p2][q][p3]] == -array[index[p1][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p3 < q && q < p4) {
					if(array[index[p3][q][p4]] == 0 || array[index[p1][q][p4]] == 0) {
						return 0;
					} else if(array[index[p3][q][p4]] != 0 && array[index[p3][q][p4]] == -array[index[p1][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else {
					return 1;
				}
			} else if(array[index[p0][p2][p4]] == -array[index[p0][p1][p3]] && array[index[p0][p1][p3]] == array[index[p1][p3][p4]]) {
				// 1,3 UP, 2 DOWN
				if(p0 < q && q < p1) {
					if(array[index[p0][q][p1]] == 0 || array[index[p0][q][p2]] == 0) {
						return 0;
					} else if(array[index[p0][q][p1]] != 0 && array[index[p0][q][p1]] == -array[index[p0][q][p2]]) {
						return -1;
					} else {
						return 1;
					}	
				} else if(p1 < q && q < p2) {
					if(array[index[p0][q][p2]] == 0 || array[index[p1][q][p3]] == 0) {
						return 0;
					} else if(array[index[p0][q][p2]] != 0 && array[index[p0][q][p2]] == -array[index[p1][q][p3]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p2 < q && q < p3) {
					if(array[index[p1][q][p3]] == 0 || array[index[p2][q][p4]] == 0) {
						return 0;
					} else if(array[index[p1][q][p3]] != 0 && array[index[p1][q][p3]] == -array[index[p2][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p3 < q && q < p4) {
					if(array[index[p3][q][p4]] == 0 || array[index[p2][q][p4]] == 0) {
						return 0;
					} else if(array[index[p3][q][p4]] != 0 && array[index[p3][q][p4]] == -array[index[p2][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else {
					return 1;
				}
			} else if(array[index[p0][p3][p4]] == -array[index[p0][p1][p2]] && array[index[p0][p1][p2]] == array[index[p1][p2][p4]]) {
				// 1,2 UP, 3 DOWN
				if(p0 < q && q < p1) {
					if(array[index[p0][q][p1]] == 0 || array[index[p0][q][p3]] == 0) {
						return 0;
					} else if(array[index[p0][q][p1]] != 0 && array[index[p0][q][p1]] == -array[index[p0][q][p3]]) {
						return -1;
					} else {
						return 1;
					}	
				} else if(p1 < q && q < p2) {
					if(array[index[p1][q][p2]] == 0 || array[index[p0][q][p3]] == 0) {
						return 0;
					} else if(array[index[p1][q][p2]] != 0 && array[index[p1][q][p2]] == -array[index[p0][q][p3]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p2 < q && q < p3) {
					if(array[index[p0][q][p3]] == 0 || array[index[p2][q][p4]] == 0) {
						return 0;
					} else if(array[index[p0][q][p3]] != 0 && array[index[p0][q][p3]] == -array[index[p2][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else if(p3 < q && q < p4) {
					if(array[index[p3][q][p4]] == 0 || array[index[p2][q][p4]] == 0) {
						return 0;
					} else if(array[index[p3][q][p4]] != 0 && array[index[p3][q][p4]] == -array[index[p2][q][p4]]) {
						return -1;
					} else {
						return 1;
					}
				} else {
					return 1;
				}
			} else {
				return -1;
			}
		}
	} else {
		return -1;
	}
}





// fills the unassigned signs in a given partial signature function
// d = number of unassigned signs in the partial signature function
void findAllConf(int array[N], int d) {	
	if(d > 0) {
		d--;
		int aMinus[N];
		int aPlus[N];
		int dMinus = d;
		int dPlus = d;
		int firstUnassigned = -1;

		for(int i = 0; i < N; i++) {
			if(array[i] == 0 && firstUnassigned == -1) {
				firstUnassigned = i;
			}
			aMinus[i] = array[i];
			aPlus[i] = array[i];
		}
		
		int x0, x1, x2;
		int bitStack[N]; // stack of already assigned signs that need to be checked for sign forcing
		for(int i = 0; i < N; i++) {
			bitStack[i] = -1;
		}
		bitStack[0] = firstUnassigned;
		int stackSize = 1;	
	
		aMinus[firstUnassigned] = -1;
		// sign forcing
		while(stackSize > 0) {
			x0 = reverse_index[bitStack[stackSize-1]][0];
			x1 = reverse_index[bitStack[stackSize-1]][1];
			x2 = reverse_index[bitStack[stackSize-1]][2];
			stackSize--;
			int z0, z1, z2, z3;
			for(int y = 0; y < M; y++) {
				if(y != x0 && y != x1 && y != x2) {
					if(y < x0) {	
						z0 = y;	z1 = x0; z2 = x1; z3 = x2;
					} else if(x0 < y && y < x1) {
						z0 = x0; z1 = y; z2 = x1; z3 = x2;
					} else if(x1 < y && y < x2) {
						z0 = x0; z1 = x1; z2 = y; z3 = x2;
					} else if(x2 < y) {
						z0 = x0; z1 = x1; z2 = x2; z3 = y;
					}
					if(aMinus[index[z0][z1][z2]] == 0) { // 0***
						if(aMinus[index[z0][z1][z3]] != 0 && aMinus[index[z0][z1][z3]] == -aMinus[index[z0][z2][z3]]) { // 0+-*, 0-+*
							aMinus[index[z0][z1][z2]] = aMinus[index[z0][z1][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z1][z2];
							stackSize++;
						} else if(aMinus[index[z0][z2][z3]] != 0 && aMinus[index[z0][z2][z3]] == -aMinus[index[z1][z2][z3]]) { // 0*+-, 0*-+
							aMinus[index[z0][z1][z2]] = aMinus[index[z0][z2][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z1][z2];
							stackSize++;
						} else if(aMinus[index[z0][z1][z3]] != 0 && aMinus[index[z0][z1][z3]] == -aMinus[index[z1][z2][z3]]) { // 0+*-, 0-*+
							aMinus[index[z0][z1][z2]] = aMinus[index[z0][z1][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z1][z2];
							stackSize++;
						}
					}
					if(aMinus[index[z0][z1][z3]] == 0) { // *0**
						if(aMinus[index[z0][z2][z3]] != 0 && aMinus[index[z0][z2][z3]] == -aMinus[index[z1][z2][z3]]) { // *0+-, *0-+
							aMinus[index[z0][z1][z3]] = aMinus[index[z0][z2][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z1][z3];
							stackSize++;
						} else if(aMinus[index[z0][z2][z3]] != 0 && aMinus[index[z0][z2][z3]] == aMinus[index[z0][z1][z2]]) { // +0+*, -0-*
							aMinus[index[z0][z1][z3]] = aMinus[index[z0][z2][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z1][z3];
							stackSize++;
						} else if(aMinus[index[z0][z1][z2]] != 0 && aMinus[index[z0][z1][z2]] == aMinus[index[z1][z2][z3]]) { // +0*+, -0*-
							aMinus[index[z0][z1][z3]] = aMinus[index[z0][z1][z2]];
							dMinus--;
							bitStack[stackSize] = index[z0][z1][z3];
							stackSize++;
						}
					}
					if(aMinus[index[z0][z2][z3]] == 0) { // **0*
						if(aMinus[index[z0][z1][z2]] != 0 && aMinus[index[z0][z1][z2]] == -aMinus[index[z0][z1][z3]]) { // +-0*, -+0*
							aMinus[index[z0][z2][z3]] = aMinus[index[z0][z1][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z2][z3];
							stackSize++;
						} else if(aMinus[index[z0][z1][z3]] != 0 && aMinus[index[z0][z1][z3]] == aMinus[index[z1][z2][z3]]) { // *+0+, *-0-
							aMinus[index[z0][z2][z3]] = aMinus[index[z0][z1][z3]];
							dMinus--;
							bitStack[stackSize] = index[z0][z2][z3];
							stackSize++;
						} else if(aMinus[index[z0][z1][z2]] != 0 && aMinus[index[z0][z1][z2]] == aMinus[index[z1][z2][z3]]) { // +*0+, -*0-
							aMinus[index[z0][z2][z3]] = aMinus[index[z0][z1][z2]];
							dMinus--;
							bitStack[stackSize] = index[z0][z2][z3];
							stackSize++;
						}
					}
					if(aMinus[index[z1][z2][z3]] == 0) { // ***0
						if(aMinus[index[z0][z1][z3]] != 0 && aMinus[index[z0][z1][z3]] == -aMinus[index[z0][z2][z3]]) { // *+-0, *-+0
							aMinus[index[z1][z2][z3]] = aMinus[index[z0][z2][z3]];
							dMinus--;
							bitStack[stackSize] = index[z1][z2][z3];
							stackSize++;
						} else if(aMinus[index[z0][z1][z2]] != 0 && aMinus[index[z0][z1][z2]] == -aMinus[index[z0][z1][z3]]) { // +-*0, -+*0
							aMinus[index[z1][z2][z3]] = aMinus[index[z0][z1][z3]];
							dMinus--;
							bitStack[stackSize] = index[z1][z2][z3];
							stackSize++;
						} else if(aMinus[index[z0][z1][z2]] != 0 && aMinus[index[z0][z1][z2]] == -aMinus[index[z0][z2][z3]]) { // +*-0, -*+0
							aMinus[index[z1][z2][z3]] = aMinus[index[z0][z2][z3]];
							dMinus--;
							bitStack[stackSize] = index[z1][z2][z3];
							stackSize++;
						}
					}
				}		
			}
		}
		bool divided5Hole = false;
		for(x[0] = 0; x[0] < M-4; x[0]++) {
			for(x[1] = x[0]+1; x[1] < M-3; x[1]++) {
				for(x[2] = x[1]+1; x[2] < M-2; x[2]++) {
					for(x[3] = x[2]+1; x[3] < M-1; x[3]++) {
						for(x[4] = x[3]+1; x[4] < M; x[4]++) {
							bool isDivided5Hole = true;
							for(x[5] = x[0]+1; x[5] < x[4]; x[5]++) {
								if(divided5HoleCheck(aMinus, x[0], x[1], x[2], x[3], x[4], x[5]) != 1) {
									isDivided5Hole = false;
								}
							}
							if(isDivided5Hole) {
								divided5Hole = true;
							}
						}
					}
				}
			}
		}	
		if(!divided5Hole) {
			findAllConf(aMinus, dMinus);
		}
	
		for(int i = 0; i < N; i++) {
			bitStack[i] = -1;
		}
		bitStack[0] = firstUnassigned;
		stackSize = 1;
	
		aPlus[firstUnassigned]= 1;
		// sign forcing
		while(stackSize > 0) {
			x0 = reverse_index[bitStack[stackSize-1]][0];
			x1 = reverse_index[bitStack[stackSize-1]][1];
			x2 = reverse_index[bitStack[stackSize-1]][2];
			stackSize--;
			int z0, z1, z2, z3;
			for(int y = 0; y < M; y++) {
				if(y != x0 && y != x1 && y != x2) {
					if(y < x0) {	
						z0 = y; z1 = x0; z2 = x1; z3 = x2;
					} else if(x0 < y && y < x1) {
						z0 = x0; z1 = y; z2 = x1; z3 = x2;
					} else if(x1 < y && y < x2) {
						z0 = x0; z1 = x1; z2 = y; z3 = x2;
					} else if(x2 < y) {
						z0 = x0; z1 = x1; z2 = x2; z3 = y;
					}
					if(aPlus[index[z0][z1][z2]] == 0) { // 0***
						if(aPlus[index[z0][z1][z3]] != 0 && aPlus[index[z0][z1][z3]] == -aPlus[index[z0][z2][z3]]) { // 0+-*, 0-+*
							aPlus[index[z0][z1][z2]] = aPlus[index[z0][z1][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z1][z2];
							stackSize++;
						} else if(aPlus[index[z0][z2][z3]] != 0 && aPlus[index[z0][z2][z3]] == -aPlus[index[z1][z2][z3]]) { // 0*+-, 0*-+
							aPlus[index[z0][z1][z2]] = aPlus[index[z0][z2][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z1][z2];
							stackSize++;
						} else if(aPlus[index[z0][z1][z3]] != 0 && aPlus[index[z0][z1][z3]] == -aPlus[index[z1][z2][z3]]) { // 0+*-, 0-*+
							aPlus[index[z0][z1][z2]] = aPlus[index[z0][z1][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z1][z2];
							stackSize++;
						}
					}
					if(aPlus[index[z0][z1][z3]] == 0) { // *0**
						if(aPlus[index[z0][z2][z3]] != 0 && aPlus[index[z0][z2][z3]] == -aPlus[index[z1][z2][z3]]) { // *0+-, *0-+
							aPlus[index[z0][z1][z3]] = aPlus[index[z0][z2][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z1][z3];
							stackSize++;
						} else if(aPlus[index[z0][z2][z3]] != 0 && aPlus[index[z0][z2][z3]] == aPlus[index[z0][z1][z2]]) { // +0+*, -0-*
							aPlus[index[z0][z1][z3]] = aPlus[index[z0][z2][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z1][z3];
							stackSize++;
						} else if(aPlus[index[z0][z1][z2]] != 0 && aPlus[index[z0][z1][z2]] == aPlus[index[z1][z2][z3]]) { // +0*+, -0*-
							aPlus[index[z0][z1][z3]] = aPlus[index[z0][z1][z2]];
							dPlus--;
							bitStack[stackSize] = index[z0][z1][z3];
							stackSize++;
						}
					}
					if(aPlus[index[z0][z2][z3]] == 0) { // **0*
						if(aPlus[index[z0][z1][z2]] != 0 && aPlus[index[z0][z1][z2]] == -aPlus[index[z0][z1][z3]]) { // +-0*, -+0*
							aPlus[index[z0][z2][z3]] = aPlus[index[z0][z1][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z2][z3];
							stackSize++;
						} else if(aPlus[index[z0][z1][z3]] != 0 && aPlus[index[z0][z1][z3]] == aPlus[index[z1][z2][z3]]) { // *+0+, *-0-
							aPlus[index[z0][z2][z3]] = aPlus[index[z0][z1][z3]];
							dPlus--;
							bitStack[stackSize] = index[z0][z2][z3];
							stackSize++;
						} else if(aPlus[index[z0][z1][z2]] != 0 && aPlus[index[z0][z1][z2]] == aPlus[index[z1][z2][z3]]) { // +*0+, -*0-
							aPlus[index[z0][z2][z3]] = aPlus[index[z0][z1][z2]];
							dPlus--;
							bitStack[stackSize] = index[z0][z2][z3];
							stackSize++;
						}
					}
					if(aPlus[index[z1][z2][z3]] == 0) { // ***0
						if(aPlus[index[z0][z1][z3]] != 0 && aPlus[index[z0][z1][z3]] == -aPlus[index[z0][z2][z3]]) { // *+-0, *-+0
							aPlus[index[z1][z2][z3]] = aPlus[index[z0][z2][z3]];
							dPlus--;
							bitStack[stackSize] = index[z1][z2][z3];
							stackSize++;
						} else if(aPlus[index[z0][z1][z2]] != 0 && aPlus[index[z0][z1][z2]] == -aPlus[index[z0][z1][z3]]) { // +-*0, -+*0
							aPlus[index[z1][z2][z3]] = aPlus[index[z0][z1][z3]];
							dPlus--;
							bitStack[stackSize] = index[z1][z2][z3];
							stackSize++;
						} else if(aPlus[index[z0][z1][z2]] != 0 && aPlus[index[z0][z1][z2]] == -aPlus[index[z0][z2][z3]]) { // +*-0, -*+0
							aPlus[index[z1][z2][z3]] = aPlus[index[z0][z2][z3]];
							dPlus--;
							bitStack[stackSize] = index[z1][z2][z3];
							stackSize++;
						}
					}
				}		
			}
		}
		divided5Hole = false;
		for(x[0] = 0; x[0] < M-4; x[0]++) {
			for(x[1] = x[0]+1; x[1] < M-3; x[1]++) {
				for(x[2] = x[1]+1; x[2] < M-2; x[2]++) {
					for(x[3] = x[2]+1; x[3] < M-1; x[3]++) {
						for(x[4] = x[3]+1; x[4] < M; x[4]++) {
							bool isDivided5Hole = true;
							for(x[5] = x[0]+1; x[5] < x[4]; x[5]++) {
								if(divided5HoleCheck(aPlus, x[0], x[1], x[2], x[3], x[4], x[5]) != 1) {
									isDivided5Hole = false;
								}
							}
							if(isDivided5Hole) {
								divided5Hole = true;
							}
						}
					}
				}
			}
		}	
		if(!divided5Hole) {
			findAllConf(aPlus, dPlus);
		}
	} else {
		// checks whether the signature function is monotone (that is, it contains only 4-tuples +-++, ++-+, -+--, --+-, +-+-, -+-+, -++-, +--+)
		bool forbidden4Tuple = false;
		int s0, s1, s2, s3;
		for(x[0] = 0; x[0] < M-3; x[0]++) {
			for(x[1] = x[0]+1; x[1] < M-2; x[1]++) {
				for(x[2] = x[1]+1; x[2] < M-1; x[2]++) {
					for(x[3] = x[2]+1; x[3] < M; x[3]++) {
		
						s0 = array[index[x[0]][x[1]][x[2]]];
						s1 = array[index[x[0]][x[1]][x[3]]];
						s2 = array[index[x[0]][x[2]][x[3]]];
						s3 = array[index[x[1]][x[2]][x[3]]];

						if(s0 == s3 && s1 == -s2) {
							// ++-+, +-++, -+--, --+-
							forbidden4Tuple = true;
						}
						if(s0 == -s3 && s1 == -s2 && s0 == -s1) { 
							// -+-+, +-+-
							forbidden4Tuple = true; 
						}
						if(s0 == s3 && s1 == s2 && s0 == -s1) {
							// +--+, -++-
							forbidden4Tuple = true;
						}	
					}
				}
			}
		}

		// checks whether there is an l-divided 5-hole in P
		bool divided5Hole = false;
		for(x[0] = 0; x[0] < M-4; x[0]++) {
			for(x[1] = x[0]+1; x[1] < M-3; x[1]++) {
				for(x[2] = x[1]+1; x[2] < M-2; x[2]++) {
					for(x[3] = x[2]+1; x[3] < M-1; x[3]++) {
						for(x[4] = x[3]+1; x[4] < M; x[4]++) {
							bool isDivided5Hole = true;
							for(x[5] = x[0]+1; x[5] < x[4]; x[5]++) {
								if(divided5HoleCheck(array, x[0], x[1], x[2], x[3], x[4], x[5]) == -1) {
									isDivided5Hole = false;
								}
							}
							if(isDivided5Hole) {
								divided5Hole = true;
							}
						}
					}
				}
			}
		}

		if(!forbidden4Tuple && !divided5Hole) {
			cout << "There is a signature function P with A not in convex position and without an l-divided 5-hole in P.\n";
			exit(0);
		}
	}
}



// reads the pre-computed signature functions for A
void readA() {
	FILE *stream;
	int d;

	stream = fopen(filenameA, "r");
	if( stream == NULL ) {
		printf("\nThe file was not opened.\n");
		exit(0);
	}

	for(int a = 0; a < numberOfConfA; a++) {
		for(int j = 0; j < NA; j++) {
			fscanf(stream, "%d", &d);
			if(d == 0) {
				d = -1;
			}
			confA[a][j] = d;
		}
	}
}



// reads the pre-computed signature functions for B
void readB() {
	FILE *stream;
	int d;

	stream = fopen(filenameB, "r");
	if( stream == NULL ) {
		printf("\nThe file was not opened.\n");
		exit(0);
	}

	for(int b = 0; b < numberOfConfB; b++) {
		for(int j = 0; j < NB; j++) {
			fscanf(stream, "%d", &d);
			if(d == 0) {
				d = -1;
			}
			confB[b][j] = d;
		}
	}
}


	

// sets indexations for triples of points of A, B, and P
void set_index() {
	// for P
	int n = 0;
	for(int i = 0; i < M-2; i++) {
		for(int j = i+1; j < M-1; j++) {
			for(int k = j+1; k < M; k++) {
				index[i][j][k] = n;
				reverse_index[n][0] = i;
				reverse_index[n][1] = j;
				reverse_index[n][2] = k;
				n++;
			}
		}
	}

	// for A
	n = 0;
	for(int i = 0; i < MA-2; i++) {
		for(int j = i+1; j < MA-1; j++) {
			for(int k = j+1; k < MA; k++) {
				indexA[i][j][k] = n;
				reverse_indexA[n][0] = i;
				reverse_indexA[n][1] = j;
				reverse_indexA[n][2] = k;
				n++;
			}
		}
	}

	// for B
	n = 0;
	for(int i = 0; i < MB-2; i++) {
		for(int j = i+1; j < MB-1; j++) {
			for(int k = j+1; k < MB; k++) {
				indexB[i][j][k] = n;
				reverse_indexB[n][0] = i;
				reverse_indexB[n][1] = j;
				reverse_indexB[n][2] = k;
				n++;
			}
		}
	}
}



int main() {
	if(!(start < end) || !(start >= 0) || !(end <= numberOfConfA)) {
		exit(0);
	}
	set_index();
	int orderType[N];
	readA();
	readB();
	// goes through all pairs (A,B), fills the unassigned signs using a depth-first search algorithm, and checks l-divided 5-holes in P
	for(int a = start; a < end; a++) {
		for(int b = 0; b < numberOfConfB; b++) {
			cout << "Checking pair (A,B) number " << a*numberOfConfB + b + 1 << " (out of " << numberOfConfA*numberOfConfB << ").\n";

			for(int i = 0; i < N; i++) {
				orderType[i] = 0;
			}
			for(int i = 0; i < MA-2; i++) {
				for(int j = i+1; j < MA-1; j++) {
					for(int k = j+1; k < MA; k++) {
						orderType[index[i][j][k]] = confA[a][indexA[i][j][k]];
					}
				}				
			}
			for(int i = 0; i < MB-2; i++) {
				for(int j = i+1; j < MB-1; j++) {
					for(int k = j+1; k < MB; k++) {
						orderType[index[i+MA][j+MA][k+MA]] = confB[b][indexB[i][j][k]];
					}
				}				
			}
			findAllConf(orderType,N-NA-NB);
		}
	}
	cout << "\n\n\nThere is no l-divided set P=(A,B) with |A|=5, |B|=6, A not in convex position, and with no l-divided 5-hole in P.\n";
	cout << "We checked signature functions for A in the range " << start << " -- " <<  end << ".\n";

	return 0;
}
